package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.enums.EnumTipoVenda;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

public class RelatorioEstoque extends Relatorio {

	public RelatorioEstoque(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new ArrayDataSource(this) {

			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				String sql = getSql();

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

				ResultSet rset = stmt.executeQuery();

				while (rset.next()) {
					Map<String, Object> dataResult = new HashMap<String, Object>();

					String tipoVenta = rset.getString("tipoVenta");
					String formfinal = rset.getString("formfinal");

					String formInicio = rset.getString("forminicial");
					if (tipoVenta.equals(EnumTipoVenda.TPV_MANUAL.getId().toString())) {

						String ultimoFolioCaja = rset.getString("ultimoFolioCaja");

						boolean isFormularioMaior = false;
						try {
							int ultimoFolioCajaInt = Integer.parseInt(ultimoFolioCaja);
							int formfinalInt = Integer.parseInt(formfinal);

							if (ultimoFolioCaja != null && ultimoFolioCajaInt > formfinalInt) {
								isFormularioMaior = true;
							}

						} catch (Exception e) {
						}

						if (isFormularioMaior) {
							formInicio = formfinal;
						} else {
							formInicio = ultimoFolioCaja != null ? ultimoFolioCaja : formInicio;
						}
					}

					dataResult.put("aidf", rset.getString("aidf"));
					dataResult.put("empresa", rset.getString("nombempresa"));
					dataResult.put("estacao", rset.getString("estacao"));
					dataResult.put("agencia", rset.getString("nombpuntoventa"));
					dataResult.put("serie", rset.getString("serie"));
					dataResult.put("forminicio", formInicio);
					dataResult.put("formfinal", formfinal);
					dataResult.put("estado", rset.getString("nombestado"));
					dataResult.put("tipo", rset.getString("tipo"));
					this.dados.add(dataResult);
				}

				this.resultSet = rset;
			}
		});
	}

	@Override
	protected void processaParametros() throws Exception {
	}

	private String getSql() {
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT dab.aidf_id AS aidf, ");
		sql.append("	e.nombempresa, ");
		sql.append("	p.nombpuntoventa, ");
		sql.append("	dab.NUMSERIEPREIMPRESA AS serie, ");
		sql.append("	es.descestacion AS estacao, ");
		sql.append("	tp.desctipo AS tipo, ");

		sql.append(" 	( case  ");
		sql.append("    	 when ");
		sql.append("			dab.STATUSOPERACION = 0 ");
		sql.append("     	then  ");
		sql.append("			dab.NUMFOLIOINICIAL");
		sql.append("     	else ");
		sql.append("     		coalesce(fp.FOLIOPREIMPRESO,dab.NUMFOLIOINICIAL) ");
		sql.append(" 	end ) AS forminicial,");
		sql.append(" 	dab.NUMFOLIOFINAL AS formfinal, ");

		sql.append("	( SELECT ");
		sql.append(" 		MAX(to_number(c.numfoliopreimpreso)) + 1 AS numfoliopreimpreso");
		sql.append(" 		 FROM Caja c");
		sql.append("	WHERE  ( ( c.numseriepreimpresa = dab.numseriepreimpresa ");
		sql.append("				 AND c.aidf_id = dab.aidf_id ) ");
		sql.append(" 			OR ( c.numseriepreimpresa IS NULL ");
		sql.append(" 				 AND dab.numseriepreimpresa IS NULL ) ) ");
		sql.append(" 			AND to_number(c.numfoliopreimpreso) BETWEEN dab.numfolioinicial AND dab.numfoliofinal ");
		sql.append(" 			AND ( c.indreimpresion = 0 ");
		sql.append(" 			OR ( c.indreimpresion = 1 AND c.indstatusboleto = 'E' ) ) ");
		sql.append("			AND c.marca_id = ab.empresa_id ");
		sql.append("	) AS ultimoFolioCaja, ");

		sql.append(" 	ar.articulo_id  AS tipoventa,");
		sql.append(" 	ar.descarticulo, ");
		sql.append(" 	est.nombestado ");
		sql.append(" FROM abasto_boleto ab ");
		sql.append(" 	INNER JOIN empresa e ON ( e.empresa_id = ab.empresa_id and ab.ACTIVO = 1 )");
		sql.append(" 	INNER JOIN det_abasto_boleto dab ON ( dab.abastoboleto_id = ab.abastoboleto_id and dab.ACTIVO = 1 )");
		sql.append(" 	INNER JOIN punto_venta p ON ( p.puntoventa_id = ab.puntoventa_id and p.ACTIVO = 1 )");
		sql.append(" 	LEFT JOIN  TIPO_PTOVTA tp ON ( tp.TIPOPTOVTA_ID = p.TIPOPTOVTA_ID and tp.ACTIVO = 1 )");
		sql.append(" 	INNER JOIN estacion es ON ( es.ESTACION_ID = ab.ESTACION_ID )");
		sql.append(" 	INNER JOIN ARTICULO ar ON ( ar.ARTICULO_ID = ab.ARTICULO_ID )");
		sql.append(" 	LEFT JOIN  FOLIO_PREIMPRESO fp ON ( fp.AIDF_ID = dab.AIDF_ID and fp.EMPRESA_ID = e.EMPRESA_ID and fp.ESTACION_ID = ab.ESTACION_ID and fp.activo = 1 )");
		sql.append(" 	INNER JOIN aidf ai ON ( ai.aidf_id =dab.aidf_id )");
		sql.append(" 	INNER JOIN estado est ON ( est.estado_id = ai.ESTADO_ID ) ");

		sql.append(" WHERE ");
		sql.append(" 	dab.statusoperacion in (0,1) and dab.activo = 1 and ab.activo = 1 ");
		if (parametros.get("EMPRESA_ID") != null) {
			sql.append(" AND e.empresa_id = ").append(parametros.get("EMPRESA_ID"));
		}
		if (parametros.get("NUMPUNTOVENTA") != null) {
			sql.append(" AND ab.puntoventa_id in(").append(parametros.get("NUMPUNTOVENTA")).append(") ");
		}
		sql.append(" ORDER BY e.nombempresa, p.nombpuntoventa,es.descestacion, ar.descarticulo, forminicial");

		return sql.toString();

	}

}
