package com.rjconsultores.ventaboletos.relatorios.render;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.SaidaRelatorio;
import com.rjconsultores.ventaboletos.utilerias.LocaleUtil;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRParagraph;
import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.export.JExcelApiExporter;
import net.sf.jasperreports.engine.export.JExcelApiExporterParameter;
import net.sf.jasperreports.engine.export.JRXlsExporter;
import net.sf.jasperreports.engine.util.JRProperties;

/**
 * Implementao da classe abstrata de renderizao acoplada a uma tecnologia de renderizao especifica
 * 
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
@SuppressWarnings("deprecation")
public class RenderRelatorioJasper {

	private static Logger log = Logger.getLogger(Relatorio.class);

	protected JasperPrint jasperPrint;

	private Relatorio relatorio;

	/**
	 * @param relatorio
	 *            Enum do tipo Relatorio contendo as configuraes de gerao de todos os relatrios
	 * @param connection
	 *            Objeto de conexo do banco que ser utilizado para gerao do relatrio, s deve ser passado se no for utilizado nenhum tipo de Custom Data Source
	 * 
	 * @throws Exception
	 *             Excesso contendo regra de utilizao da classe
	 */
	public RenderRelatorioJasper(Relatorio relatorio) throws Exception {
		this.relatorio = relatorio;
	}

	public void preRender() {
		this.initBundler();
		this.initLocale();

	}

	private void initBundler() {
		// TODO Alterar para injeo de acordo com usurio logado ou outro meio para obter o Locale correto

		Locale locale = LocaleUtil.getLocale();
		try {

			ResourceBundle resource = ResourceBundle.getBundle("com.rjconsultores.ventaboletos.relatorios.internacionalizacao." + this.relatorio.getNome(), locale);
			this.relatorio.getParametros().put(JRParameter.REPORT_RESOURCE_BUNDLE, resource);

			/* Adicionar o arquivo de internacionalizao para o subreporte quando informado */
			if (this.relatorio.getNomeSubReporte() != null && this.relatorio.getNomeSubReporte().length > 0) {
				if (this.relatorio.getNomeSubReporte().length == 1 && StringUtils.isNotBlank(this.relatorio.getNomeSubReporte()[0])) {
					ResourceBundle resourceSubReporte = ResourceBundle.getBundle("com.rjconsultores.ventaboletos.relatorios.internacionalizacao." + this.relatorio.getNomeSubReporte()[0], locale);
					this.relatorio.getParametros().put("SUBREPORT_RESOURCE_BUNDLE", resourceSubReporte);
				} else if (this.relatorio.getNomeSubReporte() != null && this.relatorio.getNomeSubReporte().length > 1) {
					int indice = 0;
					for (String nomeSubReporte : this.relatorio.getNomeSubReporte()) {
						if (StringUtils.isNotBlank(nomeSubReporte)) {
							ResourceBundle resourceSubReporte = ResourceBundle.getBundle("com.rjconsultores.ventaboletos.relatorios.internacionalizacao." + nomeSubReporte, locale);
							this.relatorio.getParametros().put("SUBREPORT_RESOURCE_BUNDLE" + "_" + (++indice), resourceSubReporte);
						}
					}
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
	}

	private void initLocale() {
		this.relatorio.getParametros().put(JRParameter.REPORT_LOCALE, LocaleUtil.getLocale());
	}

	protected InputStream getTemplateInputStream() throws Exception {
		String diretorio = "/com/rjconsultores/ventaboletos/relatorios/templates/" + this.relatorio.getNome() + ".jasper";
		InputStream iStemplate = this.getClass().getResourceAsStream(diretorio);

		if (iStemplate == null) {
			throw new Exception("No foi possivel localizar o template do reltorio no diretorio a seguir: " + diretorio);
		}

		return iStemplate;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.IRenderRelatorio#render()
	 */

	public byte[] render(SaidaRelatorio saida, boolean isCustomXls) throws Exception {
		this.preRender();

		byte[] conteudo = null;
		try {
			if (this.jasperPrint == null) {

				if (this.relatorio.isCollectionDataSource()) {
					if (!StringUtils.isBlank(this.relatorio.getNome()) && this.relatorio.getNome().equalsIgnoreCase("RelatorioGratuidade")) {
						JRProperties.setProperty(JRParagraph.DEFAULT_TAB_STOP_WIDTH, "10");
					}
					this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), this.relatorio.getCollectionDataSource());

				} else {
					if (this.relatorio.isCustomDataSource())
						this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), (JRDataSource) this.relatorio.getCustomDataSource());
					else
						this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), this.relatorio.getConexao());
				}
			}

			switch (saida) {

			case PDF:
				conteudo = this.renderPdf();
				break;

			case XLS:
				conteudo = this.renderXls(isCustomXls);
				break;

			}

		} finally {
			if (!StringUtils.isBlank(this.relatorio.getNome()) && this.relatorio.getNome().equalsIgnoreCase("RelatorioGratuidade")) {
				JRProperties.setProperty(JRParagraph.DEFAULT_TAB_STOP_WIDTH, "40");
			}
		}

		return conteudo;
	}

	protected byte[] renderXls(boolean isCustomXls) throws Exception {
		JExcelApiExporter exporterXLS = new JExcelApiExporter();
		ByteArrayOutputStream output = new ByteArrayOutputStream();

		// As duas propriedades so para quando o arquivo xls chegar no mximo de linhas pular para a prxima aba do arquivo mantis #11294
		exporterXLS.setParameter(JExcelApiExporterParameter.IS_ONE_PAGE_PER_SHEET, Boolean.TRUE);
		exporterXLS.setParameter(JExcelApiExporterParameter.MAXIMUM_ROWS_PER_SHEET, 65500);

		exporterXLS.setParameter(JExcelApiExporterParameter.JASPER_PRINT, this.jasperPrint);
		exporterXLS.setParameter(JExcelApiExporterParameter.OUTPUT_STREAM, output);

		if (!isCustomXls) {
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_ONE_PAGE_PER_SHEET, Boolean.FALSE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_DETECT_CELL_TYPE, Boolean.TRUE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_WHITE_PAGE_BACKGROUND, Boolean.FALSE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_ROWS, Boolean.TRUE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_COLUMNS, Boolean.TRUE);
		}

		// collapseRowSpan
		// JRXlsExporterParameter.
		exporterXLS.exportReport();
		return output.toByteArray();
	}
	
	protected byte[] renderXlsNovo(boolean isCustomXls) throws Exception {
		JRXlsExporter exporterXLS = new JRXlsExporter();
		ByteArrayOutputStream output = new ByteArrayOutputStream();
/*
		// As duas propriedades so para quando o arquivo xls chegar no mximo de linhas pular para a prxima aba do arquivo mantis #11294
		exporterXLS.setParameter(JRXlsExporterParameter.IS_ONE_PAGE_PER_SHEET, Boolean.TRUE);
		exporterXLS.setParameter(JRXlsExporterParameter.MAXIMUM_ROWS_PER_SHEET, 65500);

		exporterXLS.setParameter(XlsExporterConfiguration.JASPER_PRINT, this.jasperPrint);
		exporterXLS.setParameter(JRXlsExporterParameter.OUTPUT_STREAM, output);
		
		exporterXLS.setExporterInput(new SimpleExporterInput(jasperPrint));
		exporterXLS.setExporterOutput(new SimpleOutputStreamExporterOutput(destFile));
		SimpleXlsReportConfiguration configuration = new SimpleXlsReportConfiguration();
		configuration.setOnePagePerSheet(false);
		configuration.setRemoveEmptySpaceBetweenRows(true);
		exporter.setConfiguration(configuration);

		if (!isCustomXls) {
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_ONE_PAGE_PER_SHEET, Boolean.FALSE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_DETECT_CELL_TYPE, Boolean.TRUE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_WHITE_PAGE_BACKGROUND, Boolean.FALSE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_ROWS, Boolean.TRUE);
			exporterXLS.setParameter(JExcelApiExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_COLUMNS, Boolean.TRUE);
		}
*/
		// collapseRowSpan
		// JRXlsExporterParameter.
		exporterXLS.exportReport();
		return output.toByteArray();
	}

	protected byte[] renderPdf() throws Exception {
		return JasperExportManager.exportReportToPdf(this.jasperPrint);
	}

}
