package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Radio;

import com.rjconsultores.ventaboletos.constantes.Constantes.FiltroEnviadosSefaz;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioVendaEmbarcada;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxPuntoVenta;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;

@Controller("relatorioVendaEmbarcadaController")
@Scope("prototype")
public class RelatorioVendaEmbarcadaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger(RelatorioVendaEmbarcadaController.class);
	
	@Autowired
	private DataSource dataSourceRead;
	@Autowired
	private EstadoService estadoService;
	
	private MyComboboxEmpresa cmbEmpresa;
	private MyComboboxPuntoVenta cmbPuntoVenta;
	private Datebox datInicial;
	private Datebox datFinal;
	private Checkbox chkBloqueado;
	private Radio radEnvioSim;
	private Radio radEnvioNao;
	private Radio radEnvioAmbos;
	private Radio radVendaEmbarcadaSim;
	private Radio radVendaEmbarcadaNao;
	private Radio radVendaEmbarcadaAmbos;
	private MyComboboxEstandar cmbEstado;
	private Checkbox chkQuebraSequencia;
	private MyTextbox txtSerie;
	private Checkbox chkUltimoBpe;
	private MyTextbox txtNumBpe;
	
	private Integer estadoId;
	private Integer empresaId;
	private Integer puntoVendaId;
	private String serie;
	private String numBpe;
	private Boolean bloqueado;
	private FiltroEnviadosSefaz envioSefaz;
	private Boolean quebraSequencia;
	private Boolean ultimoBpe;
	
	private List<Estado> lsEstados;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEstados = estadoService.obtenerTodos();	
		super.doAfterCompose(comp);
	}
	
	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio() throws Exception {

		//Map<String, Object> parametros = new HashMap<String, Object>();
		StringBuilder filtro = new StringBuilder();

		Map<String, Object> parametros = new HashMap<String, Object>();
		parametros.put("TITULO", Labels.getLabel("relatorioVendaEmbarcadaController.window.title"));
		parametros.put("NOME_RELATORIO", Labels.getLabel("relatorioVendaEmbarcadaController.window.title"));
		parametros.put("USUARIO_ID", UsuarioLogado.getUsuarioLogado().getUsuarioId().toString());
		parametros.put("USUARIO", UsuarioLogado.getUsuarioLogado().getNombusuario());
		parametros.put("DATA_INICIAL", datInicial.getValue());
		parametros.put("DATA_FINAL", datFinal.getValue());
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			empresaId = empresa.getEmpresaId();
			parametros.put("EMPRESA", empresa.getNombempresa());
			parametros.put("EMPRESA_ID", empresa.getEmpresaId());
		}
		
		Comboitem itemPuntoVenta = cmbPuntoVenta.getSelectedItem();
		if (itemPuntoVenta != null) {
			PuntoVenta ptovta = (PuntoVenta) itemPuntoVenta.getValue();
			puntoVendaId = ptovta.getPuntoventaId();
			parametros.put("PUNTOVENTA", ptovta.getNombpuntoventa());
			parametros.put("PUNTOVENTA_ID", ptovta.getPuntoventaId());
		}

		Comboitem itemEstado = cmbEstado.getSelectedItem();
		if (itemEstado != null) {
			Estado estado = (Estado) itemEstado.getValue();
			estadoId = estado.getEstadoId();
			parametros.put("ESTADO", estado.getNombestado());
			parametros.put("ESTADO_ID", estado.getEstadoId());
		}
		
		if (radEnvioAmbos.isChecked()){
			parametros.put("ENVIOSEFAZ", FiltroEnviadosSefaz.TODOS);
		} else if (radEnvioNao.isChecked()){
			parametros.put("ENVIOSEFAZ", FiltroEnviadosSefaz.NAO);
		} else {
			parametros.put("ENVIOSEFAZ", FiltroEnviadosSefaz.SIM);
		}
		
		if (radVendaEmbarcadaAmbos.isChecked()){
			parametros.put("VENDAEMBARCADA", FiltroEnviadosSefaz.TODOS);
		} else if (radVendaEmbarcadaNao.isChecked()){
			parametros.put("VENDAEMBARCADA", FiltroEnviadosSefaz.NAO);
		} else {
			parametros.put("VENDAEMBARCADA", FiltroEnviadosSefaz.SIM);
		}
		
		parametros.put("QUEBRASEQ", chkQuebraSequencia.isChecked());
		
		SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");
		filtro.append(" DATA: " + df.format(new Date()));
		parametros.put("DATA", new Date());
		
		serie = txtSerie.getText();
		parametros.put("SERIE", serie);
		numBpe = txtNumBpe.getText();
		parametros.put("NUMBPE", numBpe);
		bloqueado = chkBloqueado.isChecked();
		parametros.put("BLOQUEADO", bloqueado);
		quebraSequencia = chkQuebraSequencia.isChecked();
		ultimoBpe = chkUltimoBpe.isChecked();
		parametros.put("ULTIMOBPE", ultimoBpe);
		
		Relatorio relatorio = new RelatorioVendaEmbarcada(parametros, dataSourceRead.getConnection());
		
		Map<String, Object> args = new HashMap<String, Object>();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				Labels.getLabel("relatorioVendaEmbarcadaController.window.title"), args, MODAL);
	}
	
	public EstadoService getEstadoService() {
		return estadoService;
	}

	public void setEstadoService(EstadoService estadoService) {
		this.estadoService = estadoService;
	}

	public MyComboboxEmpresa getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEmpresa cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyComboboxPuntoVenta getCmbPuntoVenta() {
		return cmbPuntoVenta;
	}

	public void setCmbPuntoVenta(MyComboboxPuntoVenta cmbPuntoVenta) {
		this.cmbPuntoVenta = cmbPuntoVenta;
	}

	public Datebox getDatInicial() {
		return datInicial;
	}

	public void setDatInicial(Datebox datInicial) {
		this.datInicial = datInicial;
	}

	public Datebox getDatFinal() {
		return datFinal;
	}

	public void setDatFinal(Datebox datFinal) {
		this.datFinal = datFinal;
	}

	public List<Estado> getLsEstados() {
		return lsEstados;
	}

	public void setLsEstados(List<Estado> lsEstados) {
		this.lsEstados = lsEstados;
	}

	public Checkbox getChkBloqueado() {
		return chkBloqueado;
	}

	public void setChkBloqueado(Checkbox chkBloqueado) {
		this.chkBloqueado = chkBloqueado;
	}

	public Radio getRadEnvioSim() {
		return radEnvioSim;
	}

	public void setRadEnvioSim(Radio radEnvioSim) {
		this.radEnvioSim = radEnvioSim;
	}

	public Radio getRadEnvioNao() {
		return radEnvioNao;
	}

	public void setRadEnvioNao(Radio radEnvioNao) {
		this.radEnvioNao = radEnvioNao;
	}

	public Radio getRadEnvioAmbos() {
		return radEnvioAmbos;
	}

	public void setRadEnvioAmbos(Radio radEnvioAmbos) {
		this.radEnvioAmbos = radEnvioAmbos;
	}

	public MyComboboxEstandar getCmbEstado() {
		return cmbEstado;
	}

	public void setCmbEstado(MyComboboxEstandar cmbEstado) {
		this.cmbEstado = cmbEstado;
	}

	public Checkbox getChkQuebraSequencia() {
		return chkQuebraSequencia;
	}

	public void setChkQuebraSequencia(Checkbox chkQuebraSequencia) {
		this.chkQuebraSequencia = chkQuebraSequencia;
	}

	public MyTextbox getTxtSerie() {
		return txtSerie;
	}

	public void setTxtSerie(MyTextbox txtSerie) {
		this.txtSerie = txtSerie;
	}

	public Checkbox getChkUltimoBpe() {
		return chkUltimoBpe;
	}

	public void setChkUltimoBpe(Checkbox chkUltimoBpe) {
		this.chkUltimoBpe = chkUltimoBpe;
	}

	public MyTextbox getTxtNumBpe() {
		return txtNumBpe;
	}

	public void setTxtNumBpe(MyTextbox txtNumBpe) {
		this.txtNumBpe = txtNumBpe;
	}

	public Radio getRadVendaEmbarcadaSim() {
		return radVendaEmbarcadaSim;
	}

	public void setRadVendaEmbarcadaSim(Radio radVendaEmbarcadaSim) {
		this.radVendaEmbarcadaSim = radVendaEmbarcadaSim;
	}

	public Radio getRadVendaEmbarcadaNao() {
		return radVendaEmbarcadaNao;
	}

	public void setRadVendaEmbarcadaNao(Radio radVendaEmbarcadaNao) {
		this.radVendaEmbarcadaNao = radVendaEmbarcadaNao;
	}

	public Radio getRadVendaEmbarcadaAmbos() {
		return radVendaEmbarcadaAmbos;
	}

	public void setRadVendaEmbarcadaAmbos(Radio radVendaEmbarcadaAmbos) {
		this.radVendaEmbarcadaAmbos = radVendaEmbarcadaAmbos;
	}

}
